<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Block\Adminhtml\Quotes;

/**
 * Class Grid
 *
 * @package Propoza\Propoza\Block\Adminhtml\Quotes
 */
class Grid extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;
    /**
     * @var \Magento\Backend\Model\Auth\Session
     */
    private $authSession;
    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    private $curl;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;

    /**
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer
     * @param \Magento\Backend\Model\Auth\Session $authSession
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer,
        \Magento\Backend\Model\Auth\Session $authSession,
        \Magento\Framework\View\Element\Template\Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->curl = $curl;
        $this->propozaHelper = $propozaHelper;
        $this->authSession = $authSession;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * @return string
     */
    public function getDashboardPropozaLoginUrl()
    {
        return $this->propozaHelper->getDashboardPropozaLoginUrl(null, $this->getPropozaDashboardToken());
    }

    /**
     * Get the propoza dashboard login token
     * @return string
     */
    public function getPropozaDashboardToken()
    {
        if (!$token = $this->authSession->getPropozaDashboardToken()) {
            // @codingStandardsIgnoreLine
            $api_key = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
            $this->curl->setCredentials($api_key[0], $api_key[1]);
            $this->curl->addHeader('Content-Type', 'application/json');
            $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
            if ($this->propozaHelper->isDeveloperMode()) {
                $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
                $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
            }
            $this->curl->get($this->propozaHelper->getDashboardPropozaTokenUrl());
            $response = $this->jsonSerializer->unserialize($this->curl->getBody());

            if (isset($response['response']['token'])) {
                $this->authSession->setPropozaDashboardToken($response['response']['token']);
            }
        }

        return $token;
    }
}
