<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Block\Adminhtml\System\Config\Button;

/**
 * Class TestConnection
 *
 * @package Propoza\Propoza\Block\Adminhtml\System\Config\Button
 */
class TestConnection extends \Magento\Config\Block\System\Config\Form\Field
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;

    /**
     * TestConnection constructor.
     *
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Magento\Backend\Block\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->propozaHelper = $propozaHelper;
    }

    /**
     * Unset some non-related element parameters
     *
     * @param  \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();

        return parent::render($element);
    }

    /**
     * Set template to itself
     *
     * @return \Propoza\Propoza\Block\Adminhtml\System\Config\Button\TestConnection
     */
    // @codingStandardsIgnoreLine
    /**
     * @return string
     */
    public function getTestConnectionControllerActionUrl()
    {
        return $this->getUrl('propoza/ajax/testconnection');
    }

    /**
     * @return string
     */
    public function getDashboardPropozaUrl()
    {
        return $this->propozaHelper->getDashboardPropozaUrl('%s');
    }

    /**
     * @return $this
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();

        if (!$this->getTemplate()) {
            $this->setTemplate('system/config/test_connection_button.phtml');
        }

        return $this;
    }

    /**
     * Get the button contents
     *
     * @param  \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        return $this->_toHtml();
    }
}
