<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Block\Quote\Request;

use Magento\Framework\Exception\AuthenticationException;

/**
 * Class Button
 *
 * @package Propoza\Propoza\Block\Quote\Request
 */
class Button extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    protected $curl;

    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    protected $propozaHelper;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var \Magento\Checkout\Helper\Data
     */
    private $checkoutHelper;

    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    private $ruleHelper;

    /**
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Checkout\Helper\Data $checkoutHelper
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Checkout\Helper\Data $checkoutHelper,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->curl = $curl;
        $this->propozaHelper = $propozaHelper;
        $this->checkoutHelper = $checkoutHelper;
        $this->checkoutSession = $checkoutSession;
        $this->ruleHelper = $ruleHelper;
    }

    /**
     * * Gives back the quote button.
     * @return string of html that contains the Quote button
     * @throws AuthenticationException
     */
    public function getButton()
    {
        $apiKey = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
        $this->curl->setCredentials($apiKey[0], $apiKey[1]);
        $this->curl->addHeader('Content-Type', 'application/json');
        $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
        if ($this->propozaHelper->isDeveloperMode()) {
            $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
            $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
        }
        $this->curl->get($this->propozaHelper->getQuoteButtonUrl());
        $response = $this->curl->getBody();
        if ($this->curl->getStatus() != 200) {
            throw new AuthenticationException(__('Propoza encountered an error.'));
        }
        return $response;
    }

    /**
     * @return string
     */
    public function getRequestQuoteFormUrl()
    {
        return $this->getUrl('propoza/ajax/requestQuoteForm', ['_secure' => $this->getRequest()->isSecure()]);
    }

    /**
     * @return bool
     */
    public function isDisabled()
    {
        return !$this->checkoutSession->getQuote()->validateMinimumAmount();
    }

    /**
     * Check if the dialog and button can be shown
     *
     * @return bool
     */
    public function isEnabled()
    {
        return $this->propozaHelper->isEnabledAndConfigured() && $this->isPossibleOnepageCheckout() && !$this->isProposalQuote();
    }

    /**
     * @return bool
     */
    public function isPossibleOnepageCheckout()
    {
        return $this->checkoutHelper->canOnepageCheckout();
    }

    /**
     * Check if the rule contains a Propoza quote id
     *
     * @return mixed
     */
    public function isProposalQuote()
    {
        return $this->ruleHelper->isPropozaProposalRule($this->checkoutSession->getQuote()->getCouponCode());
    }
}
