<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Block\Quote\Request;

/**
 * Class DialogPlaceholder
 *
 * @package Propoza\Propoza\Block\Quote\Request
 */
class DialogPlaceholder extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    protected $propozaHelper;
    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Checkout\Helper\Data
     */
    protected $checkoutHelper;

    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    protected $ruleHelper;

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Checkout\Helper\Data $checkoutHelper
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Checkout\Helper\Data $checkoutHelper,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->propozaHelper = $propozaHelper;
        $this->checkoutHelper = $checkoutHelper;
        $this->checkoutSession = $checkoutSession;
        $this->ruleHelper = $ruleHelper;
    }

    /**
     * Check if the dialog and button can be shown
     *
     * @return bool
     */
    public function isEnabled()
    {
        return $this->propozaHelper->isEnabledAndConfigured() && $this->isPossibleOnepageCheckout() && !$this->isProposalQuote();
    }

    /**
     * @return bool
     */
    public function isPossibleOnepageCheckout()
    {
        return $this->checkoutHelper->canOnepageCheckout();
    }

    /**
     * Check if the rule contains a Propoza quote id
     *
     * @return mixed
     */
    public function isProposalQuote()
    {
        return $this->ruleHelper->isPropozaProposalRule($this->checkoutSession->getQuote()->getCouponCode());
    }

    /**
     * @return string
     */
    public function getRequestQuoteFormSubmitUrl()
    {
        return $this->getUrl('propoza/ajax/requestQuoteFormSubmit', ['_secure' => $this->getRequest()->isSecure()]);
    }

    public function getJavascriptUrl($relativePath)
    {
        if (strpos($relativePath, '/') !== 0) {
            $relativePath = sprintf('%s/', $relativePath);
        }
        return sprintf('%s%s', $this->propozaHelper->getDashboardPropozaUrl(), $relativePath);
    }
    /**
     * @param $type
     * @param $relativePath
     *
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function addExternalLink($type, $relativePath)
    {
        $head = $this->getLayout()->getBlock('head.additional');
        if (strpos($relativePath, '/') !== 0) {
            $relativePath = sprintf('%s/', $relativePath);
        }
        $path = sprintf('%s%s', $this->propozaHelper->getDashboardPropozaUrl(), $relativePath);

        switch ($type) {
            case 'css':
                $path = sprintf('<link rel="stylesheet" type="text/css" href="%s"/>', $path);
                break;
            case 'javascript':
                $path = sprintf('<script type="text/javascript" src="%s"></script>', $path);
                break;
        }

        $block = $this->getLayout()->createBlock('Magento\Framework\View\Element\Text')->setText($path);
        $head->append($block);

        return $this;
    }
}
