<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Block\Quote\Request;

use Magento\Framework\Exception\AuthenticationException;

/**
 * Class Button
 *
 * @package Propoza\Propoza\Block\Quote\Request
 */
class QuickQuoteButton extends \Propoza\Propoza\Block\Quote\Request\Button
{
    /**
     * @var \Magento\Framework\Registry $registry
     */
    private $_registry;

    /**
     * @param \Magento\Framework\HTTP\Client\Curl $curl is used to get the button from Propoza.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Checkout\Helper\Data $checkoutHelper
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param \Magento\Framework\Registry $registry Used to get product id.
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Checkout\Helper\Data $checkoutHelper,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        \Magento\Framework\Registry $registry,
        array $data = []
    ) {
        parent::__construct($curl, $context, $checkoutSession, $checkoutHelper, $propozaHelper, $ruleHelper);
        $this->_registry = $registry;
    }

    /**
     * * Gives back the quick quote button.
     * @return string of html that contains the QuickQuote button
     * @throws AuthenticationException
     */
    public function getButton()
    {
        $data = ['id' => $this->_registry->registry('current_product')->getId()];
        $apiKey = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
        $this->curl->setCredentials($apiKey[0], $apiKey[1]);
        $this->curl->addHeader('Content-Type', 'application/json');
        $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
        if ($this->propozaHelper->isDeveloperMode()) {
            $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
            $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
        }
        $payload = json_encode($data);
        $this->curl->post($this->propozaHelper->getQuickQuoteButtonUrl(), $payload);
        $response = $this->curl->getBody();
        if ($this->curl->getStatus() != 200) {
            throw new AuthenticationException(__('Propoza encountered an error.'));
        }

        return $response;
    }
}
