<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Controller\Adminhtml\Ajax;

use Magento\Backend\App\Action;
use Magento\Framework\Exception\AuthenticationException;
use Magento\Framework\HTTP\Client\Curl;

/**
 * Class TestConnection
 *
 * @package Propoza\Propoza\Controller\Adminhtml\Ajax
 */
class TestConnection extends \Magento\Backend\App\Action
{
    /**
     * ACL resource
     */
    const ADMIN_RESOURCE = 'Propoza_Propoza::test_connection';

    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    private $curl;

    /**
     * @var \Magento\Backend\App\Action\Context
     */
    private $context;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;

    /**
     * TestConnection constructor.
     * @param \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer
     * @param Action\Context $context
     * @param Curl $curl
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     */
    public function __construct(
        \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer,
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Propoza\Propoza\Helper\Data $propozaHelper
    ) {
        parent::__construct($context);
        $this->curl = $curl;
        $this->propozaHelper = $propozaHelper;
        $this->context = $context;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * * Test posted credentials against Propoza
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     * @throws AuthenticationException
     */
    public function execute()
    {
        $subDomainParam = $this->getRequest()->getParam('sub_domain');
        $apiKeyParam = $this->getRequest()->getParam('api_key');

        // @codingStandardsIgnoreLine
        $apiKey = explode(':', base64_decode($apiKeyParam), 2);
        $this->curl->setCredentials($apiKey[0], $apiKey[1]);
        $this->curl->addHeader('Content-Type', 'application/json');
        $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
        if ($this->propozaHelper->isDeveloperMode()) {
            $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
            $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
        }
        $this->curl->get($this->propozaHelper->getConnectionTestUrl($subDomainParam));
        $jsonResponse = $this->jsonSerializer->unserialize($this->curl->getBody());
        if ($this->curl->getStatus() != 200 || !isset($jsonResponse['response']) || !$jsonResponse['response']) {
            if (isset($jsonResponse['message'])) {
                throw new AuthenticationException(__('Test connection failed: %1', $jsonResponse['message']));
            }
            throw new AuthenticationException(__('Test connection failed'));
        }
        $this->getActionFlag()->set('', self::FLAG_NO_POST_DISPATCH, true);
        return $this->getResponse();
    }

    /**
     * @return bool
     */
    // @codingStandardsIgnoreLine
    protected function _isAllowed()
    {
        return parent::_isAllowed() && $this->_authorization->isAllowed(self::ADMIN_RESOURCE);
    }
}
