<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Controller\Ajax;

use Magento\Payment\Gateway\Validator\ResultInterfaceFactory;


/**
 * Class RequestQuoteFormSubmit
 *
 * @package Propoza\Propoza\Controller\Ajax
 */
class RequestQuoteForm extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;
    /**
     * @var \Magento\Framework\HTTP\Client\Curl $curl
     */
    private $curl;
    /**
     * @var \Propoza\Propoza\Helper\Request
     */
    private $requestHelper;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;

    /**
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Request $requestHelper
     * @param \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer
     * @param \Magento\Framework\App\Action\Context $context
     */
    public function __construct(
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Request $requestHelper,
        \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer,
        \Magento\Framework\App\Action\Context $context
    ) {
        parent::__construct($context);
        $this->curl = $curl;
        $this->propozaHelper = $propozaHelper;
        $this->requestHelper = $requestHelper;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        try {
            // @codingStandardsIgnoreLine
            $apiKey = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
            $this->curl->setCredentials($apiKey[0], $apiKey[1]);
            $this->curl->addHeader('Content-Type', 'text/json');
            $quote = $this->requestHelper->prepareQuoteRequest();
            $this->curl->setOption(CURLOPT_POSTFIELDS, $this->jsonSerializer->serialize($quote));
            $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
            if ($this->propozaHelper->isDeveloperMode()) {
                $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
                $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
            }
            $this->curl->post($this->propozaHelper->getQuoteRequestFormUrl(), []);
            $this->getResponse()->setBody($this->curl->getBody());
            return $this->getResponse();
        } catch (\Exception $exception) {
            return null;
        }
    }
}
