<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Controller\Checkout;

use Magento\Framework\Exception\AuthorizationException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NotFoundException;

/**
 * Class AddProposal
 *
 * @package Propoza\Propoza\Controller\Checkout
 */
class AddProposal extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;
    /**
     * @var \Magento\Quote\Model\QuoteRepository
     */
    private $quoteRepository;
    /**
     * @var \Propoza\Propoza\Helper\Quote
     */
    private $quoteHelper;
    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    private $ruleHelper;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;
    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote
     */
    private $quoteResourceModel;
    /**
     * @var \Magento\Framework\Filesystem\Io\File
     */
    private $ioFile;

    /**
     * @param \Magento\Framework\Filesystem\Io\File $ioFile
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Quote $quoteHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer
     * @param \Magento\Quote\Model\QuoteRepository $quoteRepository
     * @param \Magento\Quote\Model\ResourceModel\Quote $quoteResourceModel
     * @param \Magento\Framework\App\Action\Context $context
     */
    public function __construct(
        \Magento\Framework\Filesystem\Io\File $ioFile,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Quote $quoteHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer,
        \Magento\Quote\Model\QuoteRepository $quoteRepository,
        \Magento\Quote\Model\ResourceModel\Quote $quoteResourceModel,
        \Magento\Framework\App\Action\Context $context
    ) {
        parent::__construct($context);
        $this->propozaHelper = $propozaHelper;
        $this->quoteHelper = $quoteHelper;
        $this->ruleHelper = $ruleHelper;
        $this->quoteRepository = $quoteRepository;
        $this->jsonSerializer = $jsonSerializer;
        $this->quoteResourceModel = $quoteResourceModel;
        $this->ioFile = $ioFile;
    }

    /**
     * Add a salesrule to the quote for Propoza proposal
     *
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     * @throws AuthorizationException
     * @throws CouldNotSaveException
     * @throws NotFoundException
     * @throws \Exception
     * @throws \Magento\Framework\Exception\ValidatorException
     */
    public function execute()
    {
        if ($this->propozaHelper->isRequestAuthorized()) {
            $post = $this->jsonSerializer->unserialize($this->ioFile->read('php://input'));
            $propozaQuoteId = $post['id'];
            $quoteId = $post['shop_quote_id'];
            $proposalTotal = $post['total_proposal_price'];
            $originalTotal = $post['total_original_price'];

            $discount = $originalTotal - $proposalTotal;
            if (isset($quoteId, $proposalTotal, $originalTotal, $propozaQuoteId)) {
                $quote = $this->quoteRepository->get($quoteId);
                $clonedQuote = $this->quoteHelper->copy($quote);
                $this->quoteResourceModel->save($clonedQuote);

                $proposalRules = $this->ruleHelper->getCouponseByPropozaQuoteId($propozaQuoteId);
                if ($proposalRules->count() == 0) {
                    $couponCode = $this->ruleHelper->createProposalRule(
                        $clonedQuote,
                        $propozaQuoteId,
                        $discount,
                        $quote->getBaseSubtotal()
                    )->getPrimaryCoupon()->getCode();
                } else {
                    foreach ($proposalRules as $propozaProposalRule) {
                        $this->quoteHelper->deleteAllProposalQuotesClones(
                            $propozaProposalRule->getPrimaryCoupon()->getCode()
                        );

                        //Edit coupon to match new quote id and subtotal
                        $couponCode = $this->ruleHelper->updateProposalRule(
                            $propozaProposalRule,
                            $clonedQuote,
                            $discount,
                            $quote->getBaseSubtotal()
                        )->getPrimaryCoupon()->getCode();
                    }
                }

                //Set coupon code to proposal coupon
                if (isset($couponCode)) {
                    $clonedQuote->setCouponCode($couponCode);
                    $clonedQuote->collectTotals();
                    $this->quoteResourceModel->save($clonedQuote);
                    $this->getActionFlag()->set('', self::FLAG_NO_POST_DISPATCH, true);
                    $this->getResponse()->setHeader('Content-type', 'application/json');
                    $this->getResponse()->setBody($this->jsonSerializer->serialize($couponCode));
                } else {
                    throw new CouldNotSaveException(__('Could not save salesrule'));
                }
            } else {
                throw new NotFoundException(__('Incorrect parameters'));
            }
        } else {
            throw new AuthorizationException(__('You are not authorized to access this function'));
        }

        return $this->getResponse();
    }
}
