<?php

namespace Propoza\Propoza\Controller\Checkout;

use Magento\Customer\Api\Data\AddressInterfaceFactory;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Customer\Model\Session as CustomerSession;
use \Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Quote\Api\GuestCartRepositoryInterface;
use Magento\Quote\Model\Quote;

/**
 * Class Guest
 * @package Propoza\Propoza\Controller\Checkout
 */
class Guest extends Action
{

    /**
     * @var CustomerSession
     */
    protected $customerSession;

    /**
     * @var CheckoutSession
     */
    protected $checkoutSession;

    /**
     * @var CustomerInterfaceFactory
     */
    protected $customerInterfaceFactory;

    /**
     * @var AddressInterfaceFactory
     */
    protected $addressInterfaceFactory;

    /**
     * @var GuestCartRepositoryInterface
     */
    protected $guestCartManagement;

    /**
     * Index constructor.
     *
     * @param CheckoutSession $checkoutSession
     * @param CustomerSession $customerSession
     * @param CustomerInterfaceFactory $customerInterfaceFactory
     * @param AddressInterfaceFactory $addressInterfaceFactory
     * @param GuestCartRepositoryInterface $guestCartManagement
     * @param Context $context
     */
    public function __construct(
        CheckoutSession $checkoutSession,
        CustomerSession $customerSession,
        CustomerInterfaceFactory $customerInterfaceFactory,
        AddressInterfaceFactory $addressInterfaceFactory,
        GuestCartRepositoryInterface $guestCartManagement,
        Context $context
    ) {
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->customerInterfaceFactory = $customerInterfaceFactory;
        $this->addressInterfaceFactory = $addressInterfaceFactory;
        $this->guestCartManagement = $guestCartManagement;
        parent::__construct($context);
    }

    /**
     * Redirect to checkout
     *
     * @return \Magento\Framework\App\ResponseInterface
     * @throws \Exception
     */
    public function execute()
    {
        $guestCartId = $this->getRequest()->getParam('guest_cart');
        $propozaId = $this->getRequest()->getParam('propoza_id');
        if (!$propozaId) {
            throw new \Exception("Propoza ID is not defined");
        }

        $quote = $this->guestCartManagement->get($guestCartId);
        $this->checkoutSession->replaceQuote($quote);
        $this->checkoutSession->setPropozaQuoteId($propozaId);

        $customerData = $this->getCustomerData($quote);
        $this->customerSession->setCustomerDataObject($customerData);

        return $this->_redirect("checkout");
    }

    /**
     * @param Quote $quote
     */
    protected function getCustomerData(Quote $quote)
    {
        $billingAddress = $quote->getBillingAddress();

        $customerData = $this->customerInterfaceFactory->create();
        $customerData->setAddresses(
            [$this->generateCustomerAddress($billingAddress)]
        );

        $customerData->setEmail($billingAddress->getEmail());
        $customerData->setFirstname($billingAddress->getFirstname());
        $customerData->setLastname($billingAddress->getLastname());

        return $customerData;
    }

    /**
     * Generate customer address
     *
     * @param \Magento\Quote\Model\Quote\Address $addressData
     *
     * @return \Magento\Customer\Api\Data\AddressInterface
     */
    protected function generateCustomerAddress(\Magento\Quote\Model\Quote\Address $addressData)
    {
        $address = $this->addressInterfaceFactory->create();

        $address->setFirstname($addressData->getFirstname());
        $address->setLastname($addressData->getLastname());
        $address->setCompany($addressData->getCompany());
        $address->setStreet($addressData->getStreet());
        $address->setCity($addressData->getCity());
        $address->setPostcode($addressData->getPostcode());
        $address->setTelephone($addressData->getTelephone());
        $address->setRegion($addressData->getRegion());
        $address->setCountryId($addressData->getCountryId());

        return $address;
    }
}
