<?php

namespace Propoza\Propoza\Controller\Checkout;

use Magento\Customer\Controller\AbstractAccount;
use Magento\Customer\Model\Session as CustomerSession;
use \Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\App\Action\Context;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Quote\Model\Quote;

/**
 * Class Index
 * @package Propoza\Propoza\Controller\Checkout
 */
class Index extends AbstractAccount
{

    /**
     * @var CustomerSession
     */
    protected $customerSession;

    /**
     * @var CheckoutSession
     */
    protected $checkoutSession;

    /**
     * @var CartManagementInterface
     */
    protected $cartManagement;

    /**
     * Index constructor.
     *
     * @param CheckoutSession $checkoutSession
     * @param CustomerSession $customerSession
     * @param CartManagementInterface $cartManagement
     * @param Context $context
     */
    public function __construct(
        CheckoutSession $checkoutSession,
        CustomerSession $customerSession,
        CartManagementInterface $cartManagement,
        Context $context
    ) {
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->cartManagement = $cartManagement;
        parent::__construct($context);
    }

    /**
     * Redirect to checkout
     *
     * @return \Magento\Framework\App\ResponseInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Exception
     */
    public function execute()
    {
        $customerId = $this->customerSession->getCustomer()->getId();
        $customerCart = $this->cartManagement->getCartForCustomer($customerId);

        $propozaId = $this->getRequest()->getParam('propoza_id');
        if (!$propozaId) {
            throw new \Exception("Propoza ID is not defined");
        }

        if ($customerCart instanceof Quote) {
            $customerCart
                ->setTotalsCollectedFlag(false)
                ->collectTotals()->save();

            $this->checkoutSession->setPropozaQuoteId($propozaId);
        }

        return $this->_redirect("checkout");
    }
}
