<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Controller\Checkout;

use Magento\Framework\Exception\AuthorizationException;
use Magento\Framework\Exception\NotFoundException;


/**
 * Class Request
 *
 * @package Propoza\Propoza\Controller\Checkout
 */
class Request extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Checkout\Helper\Cart
     */
    protected $cartHelper;
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;

    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    private $ruleHelper;

    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory
     */
    private $quoteCollectionFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;
    /**
     * @var \Propoza\Propoza\Helper\Quote
     */
    private $quoteHelper;

    /**
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param \Propoza\Propoza\Helper\Quote $quoteHelper
     * @param \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Checkout\Helper\Cart $cartHelper
     * @param \Magento\Framework\App\Action\Context $context
     */
    public function __construct(
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        \Propoza\Propoza\Helper\Quote $quoteHelper,
        \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Checkout\Helper\Cart $cartHelper,
        \Magento\Framework\App\Action\Context $context
    ) {
        parent::__construct($context);
        $this->propozaHelper = $propozaHelper;
        $this->ruleHelper = $ruleHelper;
        $this->quoteCollectionFactory = $quoteCollectionFactory;
        $this->checkoutSession = $checkoutSession;
        $this->cartHelper = $cartHelper;
        $this->quoteHelper = $quoteHelper;
    }

    /**
     * Replace quote on checkout session and redirect to cart
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     * @throws AuthorizationException
     * @throws InvalidArgumentException
     * @throws NotFoundException
     */
    public function execute()
    {
        if ($this->propozaHelper->isRequestAuthorized()) {
            $couponCode = $this->getRequest()->getParam('coupon_code');
            $quote = $this->quoteHelper->getProposalQuotesClones($couponCode)->getLastItem();
            if ($quote->getId()) {
                $this->checkoutSession->replaceQuote($quote);
                if ($this->checkoutSession->getQuote()->getItemsCount() && $this->ruleHelper->isPropozaProposalRule($couponCode)) {
                    $rule = $this->ruleHelper->getSalesRuleByCouponCode($couponCode);
                    if ($rule->getId()) {
                        if ($rule->getTimesUsed() > 0) {
                            $notice = "Proposal is already used. Request a new quote for a new proposal.";
                            $this->messageManager->addNoticeMessage(
                                __('Please note: %1', $notice)
                            );
                        } else {
                            $message = "Your proposal discount has successfully been applied. Check the new Total.";
                            $this->messageManager->addSuccessMessage(
                                __('Great: %1', $message)
                            );
                            $notice = "Changing the cart contents will invalidate the proposal price.";
                            $this->messageManager->addNoticeMessage(
                                __('Please note: %1', $notice)
                            );
                        }
                    } else {
                        $this->messageManager->addErrorMessage(
                            __('Error: No proposal found for this quote.')
                        );
                    }
                } else {
                    throw new NotFoundException(__('Not a valid Propoza proposal rule'));
                }
            } else {
                throw new NotFoundException(__('Quote with this proposal coupon does not exist'));
            }
            $cartUrl = $this->cartHelper->getCartUrl();

            return $this->resultRedirectFactory->create()->setUrl($cartUrl);
        } else {
            throw new AuthorizationException(__('You are not authorized to access this function'));
        }
    }
}
