<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Controller\Quote;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Checkout\Model\Cart as CustomerCart;
use Magento\Framework\App\ObjectManager;

/**
 * Class Add
 *
 * @package Propoza\Propoza\Controller\Quote;
 */
Class Add extends \Magento\Checkout\Controller\Cart\Add
{

    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;
    /**
     * @var \Propoza\Propoza\Helper\Request
     */
    private $requestHelper;
    /**
     * @var \Magento\Framework\HTTP\Client\Curl $curl
     */
    private $curl;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;
    /**
     * @var \Propoza\Propoza\Helper\Quote
     */
    private $quoteHelper;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator
     * @param CustomerCart $cart
     * @param ProductRepositoryInterface $productRepository
     * @codeCoverageIgnore
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator,
        CustomerCart $cart,
        ProductRepositoryInterface $productRepository,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Request $requestHelper,
        \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer,
        \Propoza\Propoza\Helper\Quote $quoteHelper
    ) {
        parent::__construct(
            $context,
            $scopeConfig,
            $checkoutSession,
            $storeManager,
            $formKeyValidator,
            $cart,
            $productRepository
        );
        $this->curl = $curl;
        $this->propozaHelper = $propozaHelper;
        $this->requestHelper = $requestHelper;
        $this->jsonSerializer = $jsonSerializer;
        $this->quoteHelper = $quoteHelper;
    }
    /**
     * Add to our quote
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $currentQuote = $this->cart->getQuote();
        $copyQuote = $this->quoteHelper->copy($currentQuote);
        $originalCart = clone $this->cart;
        $copyQuote->clearInstance();
        $cart = ObjectManager::getInstance()->create(\Magento\Checkout\Model\Cart::class);
        $this->cart = $cart;
        $this->cart->setQuote($copyQuote);
        $this->cart->getCheckoutSession()->replaceQuote($copyQuote);
        foreach ($copyQuote->getItemsCollection() as $index => $item) {
            $this->cart->removeItem($item->getId());
            $copyQuote->getItemsCollection()->removeItemByKey($index);
        }
        parent::execute();
        $this->messageManager->getMessages(true);
        $copyQuote->collectTotals();
        $copyQuote->getResource()->save($copyQuote);
        $this->cart->getCheckoutSession()->clearQuote();
        $this->cart = $originalCart;
        $this->cart->setQuote($currentQuote);
        $this->cart->getCheckoutSession()->replaceQuote($currentQuote);
        $imageUrl = $this->getImageUrl();

        try {
            // @codingStandardsIgnoreLine
            $quote = $this->requestHelper->prepareQuoteRequest($copyQuote, $imageUrl);
            $apiKey = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
            $this->curl->setCredentials($apiKey[0], $apiKey[1]);
            $this->curl->addHeader('Content-Type', 'application/json');
            $this->curl->setOption(CURLOPT_POSTFIELDS, $this->jsonSerializer->serialize($quote));
            $this->curl->setOption(CURLOPT_FOLLOWLOCATION, true);
            if ($this->propozaHelper->isDeveloperMode()) {
                $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
                $this->curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
            }
            $this->curl->post($this->propozaHelper->getQuickQuoteUrl(), []);
            $this->getResponse()->setBody($this->curl->getBody());
            return $this->getResponse();
        } catch (\Exception $exception) {
            return null;
        }

    }

    private function getImageUrl()
    {
        $productId = $this->getRequest()->getParam('product');
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $product = $objectManager->create('Magento\Catalog\Model\Product')->load($productId);
        $store = $objectManager->get('Magento\Store\Model\StoreManagerInterface')->getStore();
        $imageUrl = $store->getBaseUrl(
                \Magento\Framework\UrlInterface::URL_TYPE_MEDIA
            ) . 'catalog/product' . $product->getImage();

        return $imageUrl;
    }
}