<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Helper\Json;


/**
 * Class Serializer
 * @package Propoza\Propoza\Helper\Json
 */
class Serializer extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\Serialize\Serializer\Json
     */
    private $jsonSerializer;

    /**
     * @var \Magento\Framework\Json\Helper\Data
     */
    private $jsonHelper;

    /**
     * Serializer constructor.
     *
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\App\Helper\Context $context
     */
    public function __construct(
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\App\Helper\Context $context
    ) {
        parent::__construct($context);
        if (version_compare($productMetadata->getVersion(), '2.2.0') >= 0) {
            $this->jsonSerializer = $objectManager->get(\Magento\Framework\Serialize\Serializer\Json::class);
        } else {
            $this->jsonHelper = $objectManager->get(\Magento\Framework\Json\Helper\Data::class);
        }
    }

    /**
     * @param mixed $data
     *
     * @return string
     */
    public function serialize($data)
    {
        if (isset($this->jsonSerializer)) {
            return $this->jsonSerializer->serialize($data);
        }

        return $this->jsonHelper->jsonEncode($data);
    }

    /**
     * @param string $string
     * @return mixed
     */
    public function unserialize($string)
    {
        if (isset($this->jsonSerializer)) {
            return $this->jsonSerializer->unserialize($string);
        }

        return $this->jsonHelper->jsonDecode($string);
    }
}