<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Helper;

use Magento\Framework\App\Helper\Context;

/**
 * Propoza Quote helper
 */
class Quote extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory
     */
    private $quoteCollectionFactory;
    /**
     * @var \Magento\Quote\Model\QuoteFactory
     */
    private $quoteFactory;

    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    private $ruleHelper;

    /**
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param \Magento\Quote\Model\QuoteFactory $quoteFactory
     * @param \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory
     * @param Context $context
     */
    public function __construct(
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        \Magento\Quote\Model\QuoteFactory $quoteFactory,
        \Magento\Quote\Model\ResourceModel\Quote\CollectionFactory $quoteCollectionFactory,
        Context $context
    ) {
        parent::__construct($context);
        $this->ruleHelper = $ruleHelper;
        $this->quoteCollectionFactory = $quoteCollectionFactory;
        $this->quoteFactory = $quoteFactory;
    }

    /**
     * Rewrite clone method
     *
     * @param  \Magento\Quote\Model\Quote $quote
     * @param  boolean $quickQuote
     * @return \Magento\Quote\Model\Quote
     */
    public function copy($quote, $quickQuote = false)
    {
        $clone = $this->quoteFactory->create();
        $clone->setData($quote->getData());

        $clone->setId(null);

        $items = $quote->getAllItems();
        foreach ($items as $item) {
            $item->setId(null)->setQuoteId(null);
            $options = $item->getOptions();
            foreach ($options as $option) {
                $option->setOptionId(null)->setItemId(null);
            }
            $item->setOptions($options);
            $clone->addItem($item);
        }
        $addresses = $quote->getAllAddresses();
        foreach ($addresses as $key => $address) {
            $address->setId(null)->setQuoteId(null);
            $clone->addAddress($address);
        }

        foreach ($quote->getPaymentsCollection() as $payment) {
            $payment->setId(null)->setQuoteId(null);
            $clone->addPayment($payment);
        }

        return $clone;
    }

    /**
     * Delete all quotes containing propoza proposal coupon code
     *
     * @param $couponCode
     */
    public function deleteAllProposalQuotesClones($couponCode)
    {
        if ($this->ruleHelper->isPropozaProposalRule($couponCode)) {
            $this->getProposalQuotesClones($couponCode)->walk('delete');
        }
    }

    /**
     * Get all quotes that contain the coupon code
     *
     * @param  $couponCode
     * @return \Magento\Quote\Model\Quote
     */
    public function getProposalQuotesClones($couponCode)
    {
        return $this->quoteCollectionFactory->create()
            ->addFieldToFilter('coupon_code', $couponCode)
            ->addFieldToFilter('is_active', true)->load();
    }
}
