<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Class OrderPlacedObserver
 *
 * @package Propoza\Propoza\Model
 */
class OrderPlacedObserver implements ObserverInterface
{
    /**
     * @var \Propoza\Propoza\Helper\Rule
     */
    private $ruleHelper;
    /**
     * @var \Propoza\Propoza\Helper\Request
     */
    private $requestHelper;
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;

    /**
     * @var \Magento\Framework\HTTP\Client\Curl
     */
    private $curl;
    /**
     * @var \Propoza\Propoza\Helper\Quote
     */
    private $quoteHelper;
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;
    /**
     * @var \Propoza\Propoza\Helper\Json\Serializer
     */
    private $jsonSerializer;
    /**
     * @var \Magento\SalesRule\Model\ResourceModel\Rule
     */
    private $ruleResourceModel;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Propoza\Propoza\Helper\Quote $quoteHelper
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     * @param \Propoza\Propoza\Helper\Request $requestHelper
     * @param \Propoza\Propoza\Helper\Data $propozaHelper
     * @param \Propoza\Propoza\Helper\Rule $ruleHelper
     * @param \Magento\SalesRule\Model\ResourceModel\Rule $ruleResourceModel
     * @param \Propoza\Propoza\Helper\Json\Serializer $jsonSerializer
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Propoza\Propoza\Helper\Quote $quoteHelper,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Propoza\Propoza\Helper\Request $requestHelper,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Propoza\Propoza\Helper\Rule $ruleHelper,
        \Magento\SalesRule\Model\ResourceModel\Rule $ruleResourceModel,
        \Propoza\Propoza\Helper\Json\Serializer  $jsonSerializer
    ) {
        $this->logger = $logger;
        $this->quoteHelper = $quoteHelper;
        $this->curl = $curl;
        $this->ruleHelper = $ruleHelper;
        $this->requestHelper = $requestHelper;
        $this->propozaHelper = $propozaHelper;
        $this->jsonSerializer = $jsonSerializer;
        $this->ruleResourceModel = $ruleResourceModel;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @throws \Exception
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $couponCode = $observer->getEvent()->getOrder()->getCouponCode();
        $rule = $this->ruleHelper->getSalesRuleByCouponCode($couponCode);

        if ($this->ruleHelper->isPropozaProposalRule($couponCode)) {
            // @codingStandardsIgnoreLine
            $api_key = explode(':', base64_decode($this->propozaHelper->getApiKey()), 2);
            $this->curl->setCredentials($api_key[0], $api_key[1]);
            $this->curl->addHeader('Content-Type', 'application/json');
            $this->curl->setOption(
                CURLOPT_POSTFIELDS,
                $this->jsonSerializer->serialize(
                    [
                        'Quote' => [
                            'id' => $rule->getPropozaQuoteId(),
                            'main_status_id' => 2,
                            'sub_status_id' => 5,
                        ],
                    ]
                )
            );
            if ($this->propozaHelper->isDeveloperMode()) {
                $this->curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
            }
            $this->curl->post($this->propozaHelper->getQuoteOrderedUrl(), []);

            $jsonResponse = $this->jsonSerializer->unserialize($this->curl->getBody());
            if ($this->curl->getStatus() != 200 || !isset($jsonResponse['response']) || !$jsonResponse['response']) {
                $this->logger->warning(
                    __('Could not convert quote #%s to order on Propoza', $rule->getPropozaQuoteId())
                );
            }

            $this->quoteHelper->deleteAllProposalQuotesClones($couponCode);
            $this->ruleResourceModel->delete($rule);
        }
    }
}
