<?php

namespace Propoza\Propoza\Plugin;

/**
 * Class SetCustomPrice
 * @package Propoza\Propoza\Plugin
 */
class SetCustomPrice
{

    /**
     * The custom product price
     *
     * @var array
     */
    protected $productPrices = [];

    /**
     * Set the custom product price to this object
     *
     * @param $subject
     * @param $item
     *
     * @return array
     */
    public function beforeSave(
        $subject,
        $item
    ) {
        if($extension = $item->getExtensionAttributes()){
            if ($customPriceAttribute = $item->getExtensionAttributes()->getCustomPrice()) {
                $customPrice = $customPriceAttribute->getCustomPrice();
                if (isset($customPrice)) {
                    $this->productPrices[$item->getSku()] = $customPrice;
                }
            }
        }

        return [$item];
    }

    /**
     * Save the product price on the quote item and recalculate the prices
     *
     * @param $subject
     * @param $item
     *
     * @return array
     */
    public function afterSave(
        $subject,
        $item
    ) {
        if (count($this->productPrices)) {
            if (isset($this->productPrices[$item->getSku()])) {
                $itemPrice = $this->productPrices[$item->getSku()];

                $item->setCustomPrice($itemPrice);
                $item->setOriginalCustomPrice($itemPrice);
                $item->getProduct()->setIsSuperMode(true);
                $item->save();
            }
        }

        return $item;
    }
}
